//-----------------------------------------------------------------------------
// gps.cc : a simple program to test GPS class
//
//	- this program tests a GPS device
//
//	- programmed by Boyoon Jung (boyoon@robotics.usc.edu)
//-----------------------------------------------------------------------------

#include <gps.h>
#include <qgps.h>

// header files for Qt library
#include <qapplication.h>


// global settings
char serial_port[PATH_MAX] = "/dev/ttyS0";
char* dgps_server = NULL;
tcflag_t baudrate = B4800;


// process the command-line arguments
void process_arguments(int argc, char* argv[])
{
    int c;
    while ((c=getopt(argc,argv,"d:b:")) != -1)
	switch(c)
	{
	    // DGPS server
	    case 'd':
		dgps_server = strdup(optarg);
		break;

	    // baud rate
	    case 'b':
		if (!strcmp(optarg, "4800"))
		    baudrate = B4800;
		else if(!strcmp(optarg, "9600"))
		    baudrate = B9600;
		else
		    cerr << "[Error] invalid baud rate (" << optarg << ")" << endl;
		break;

	    // print help message
	    default:
		cerr << "[usage] gps [-d dgps_server] [serial_port]" << endl;
		abort();
	}

    // serial port
    if (optind < argc) {
	strcpy(serial_port, argv[optind]);
    }
}


// main function
int main(int argc, char *argv[])
{
    // initialize Qt library
    QApplication* app = new QApplication(argc, argv);

    // process the command-line arguments
    process_arguments(argc, argv);

    // initialize a GPS device
    GPS* gps;
    if (dgps_server) gps = new DGPS(dgps_server, serial_port, baudrate);
    else gps = new GPS(serial_port, baudrate);

    // construct GUI
    QGps* qgps = new QGps(gps);
    app->setMainWidget(qgps);
    qgps->show();
    app->exec();

    // done
    delete qgps;
    delete gps;
    delete app;
    return 0;
}


